#include <alt_types.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <sys/alt_flash.h>
#include "includes.h"
#include "io.h"

#include <alt_iniche_dev.h>

#include "ipport.h"
#include "tcpport.h"

#include "altera_avalon_pio_regs.h"
#include "system.h"

#define IP4_ADDR(ipaddr, a,b,c,d) ipaddr = \
    htonl((((alt_u32)(a & 0xff) << 24) | ((alt_u32)(b & 0xff) << 16) | \
          ((alt_u32)(c & 0xff) << 8) | (alt_u32)(d & 0xff)))

/*
 * get_mac_addr()
 *
 * This routine is called by InterNiche to obtain a MAC address for the
 * specified network adapter. Obtaining a MAC address is very system-dependant
 * and therefore this function is exported for the developer to control.
 *
 * In this implementation the MAC address is chosen based on the state of bit
 * ZERO of the sw_input_pio setting. Either 00-07-ED-FF-8F-10 or 00-07-ED-FF-8F-11
 * will be selected.
 */
int get_mac_addr(NET net, unsigned char mac_addr[6])
{
	    mac_addr[0] = 0x00;
	    mac_addr[1] = 0x07;
	    mac_addr[2] = 0xED;
	    mac_addr[3] = 0xFF;
	    mac_addr[4] = 0x8F;
	    if((IORD_ALTERA_AVALON_PIO_DATA(SW_BASE)) & 0x01) {
	    	mac_addr[5] = 0x11;
	    } else {
	    	mac_addr[5] = 0x10;
	    }

	    printf("Your Ethernet MAC address is %02x:%02x:%02x:%02x:%02x:%02x\n",
	            mac_addr[0],
	            mac_addr[1],
	            mac_addr[2],
	            mac_addr[3],
	            mac_addr[4],
	            mac_addr[5]);

	    return(0);
}

/*
 * get_ip_addr()
 *
 * This routine is called by InterNiche to obtain an IP address for the
 * specified network adapter. Like the MAC address, obtaining an IP address is
 * very system-dependant and therefore this function is exported for the
 * developer to control.
 *
 * In our system, we are either attempting DHCP auto-negotiation of IP address,
 * or we are setting our own static IP, Gateway, and Subnet Mask addresses our
 * self. This routine is where that happens.
 *
 * In this implementation the static IP address is chosen based on the state of
 * bit ZERO of the sw_input_pio setting. Either 10.0.0.1 or 10.0.0.2 will be
 * selected.
 */
int get_ip_addr(alt_iniche_dev *p_dev,
                ip_addr* ipaddr,
                ip_addr* netmask,
                ip_addr* gw,
                int* use_dhcp)
{

    if((IORD_ALTERA_AVALON_PIO_DATA(SW_BASE)) & 0x01) {
        IP4_ADDR(*ipaddr, 10, 0, 0, 2);
    } else {
        IP4_ADDR(*ipaddr, 10, 0, 0, 1);
    }

    IP4_ADDR(*gw, 0, 0, 0, 0);
    IP4_ADDR(*netmask, 255, 0, 0, 0);

#ifdef DHCP_CLIENT
    *use_dhcp = 1;
#else /* not DHCP_CLIENT */
    *use_dhcp = 0;

    printf("Static IP Address is %d.%d.%d.%d\n",
        ip4_addr1(*ipaddr),
        ip4_addr2(*ipaddr),
        ip4_addr3(*ipaddr),
        ip4_addr4(*ipaddr));
#endif /* not DHCP_CLIENT */

    /* Non-standard API: return 1 for success */
    return 1;
}
